import _toConsumableArray from "@babel/runtime/helpers/toConsumableArray";
import "core-js/modules/es6.array.find-index";
import "core-js/modules/es6.object.assign";
import upload from './request';
import { msgType, EStatus } from '@beisen-phoenix/upload-intf';
import { useCallback, useRef } from 'react';
import defaultParseResponse from './parseResponse';
export default function useAjaxUpload(_ref) {
  var files = _ref.files,
      url = _ref.url,
      onChange = _ref.onChange,
      onEachComplete = _ref.onEachComplete,
      onError = _ref.onError,
      _ref$parseResponse = _ref.parseResponse,
      parseResponse = _ref$parseResponse === void 0 ? defaultParseResponse : _ref$parseResponse;
  var dataInRef = useRef({
    queue: [],
    callback: onChange,
    allFiles: files,
    errCallback: onError,
    eachCallback: onEachComplete
  }); // 刷新ref下的数据，以防回调的时候执行错误的状态, 为什么把数据放到ref里呢，是因为异步回调可能导致执行错误的回调方法

  dataInRef.current = Object.assign(dataInRef.current, {
    allFiles: files,
    callback: onChange,
    eachCallback: onEachComplete,
    errCallback: onError
  });
  var updater = useCallback(function (lid, data) {
    var _dataInRef$current = dataInRef.current,
        currentFiles = _dataInRef$current.allFiles,
        currentCallback = _dataInRef$current.callback,
        currentQueue = _dataInRef$current.queue;
    var fIdx = currentFiles.findIndex(function (item) {
      return item.lid === lid;
    });

    if (fIdx === -1) {
      dataInRef.current.queue = currentQueue.filter(function (item) {
        return item !== lid;
      }); //如果没找到，删除当前队列中的任务标识

      return;
    }

    currentCallback([].concat(_toConsumableArray(currentFiles.slice(0, fIdx)), [Object.assign({}, currentFiles[fIdx], data)], _toConsumableArray(currentFiles.slice(fIdx + 1))));
  }, []); //处理上传进度的方法, 修改status为progress, 更新进度percent

  var handleProgress = useCallback(function (percent, lid) {
    updater(lid, {
      percent: percent,
      status: EStatus.progress
    });
  }, []); //上传成功后的回调

  var handleSuccess = useCallback(function (data, lid) {
    var _dataInRef$current2 = dataInRef.current,
        currentQueue = _dataInRef$current2.queue,
        eachCallback = _dataInRef$current2.eachCallback;
    var obj = Object.assign({
      status: EStatus.done
    }, parseResponse(data));

    if (currentQueue.indexOf(lid) !== -1) {
      dataInRef.current.queue = currentQueue.filter(function (item) {
        return item !== lid;
      });

      if (typeof eachCallback === 'function') {
        eachCallback(obj);
      }

      updater(lid, obj);
    }
  }, []); //错误回调

  var handleError = useCallback(function (err, lid) {
    var errCallback = dataInRef.current.errCallback;
    updater(lid, {
      status: EStatus.error
    });
    if (typeof errCallback === 'function') errCallback({
      type: msgType.networkFail,
      err: err,
      lid: lid
    });
  }, []);

  if (url) {
    files.forEach(function (f) {
      if (f.status !== EStatus.pending) return;
      var _f$lid = f.lid,
          lid = _f$lid === void 0 ? '' : _f$lid;
      if (dataInRef.current.queue.indexOf(lid) !== -1) return;
      dataInRef.current.queue.push(lid);
      var reqUrl = typeof url === 'function' ? url(f.raw) : url; //如果没有url测不执行上传

      upload({
        file: f.raw,
        url: reqUrl,
        onError: handleError,
        filename: 'file',
        onProgress: handleProgress,
        onSuccess: handleSuccess,
        withCredentials: true
      }, lid);
    });
  }
}